{% load wagtailimages_tags wagtailadmin_tags %}
{% load i18n %}
{% trans "Insert embed" as ins_emb_str %}
{% include "wagtailadmin/shared/header.html" with title=ins_emb_str merged=1 %}

<div class="tab-content">
    <section id="form" class="active nice-padding">
        <form class="embed-form" action="{% url 'wagtailembeds:chooser_upload' %}" method="POST" novalidate>
            {% csrf_token %}
            <ul class="fields">
                {% for field in form %}
                    {% include "wagtailadmin/shared/field_as_li.html" with field=field %}
                {% endfor %}
                <li><button type="submit" class="button  button-longrunning">{% icon name="spinner" %}<em>{% trans 'Insert' %}</em></button></li>
            </ul>
        </form>
    </section>
</div>
