.duplicate-upload__figure{align-items:center;display:flex;flex-direction:column}.duplicate-upload__figure img{margin:1rem auto}.duplicate-upload__actions{display:flex;justify-content:flex-end;margin-top:2.5rem}.duplicate-upload__actions .use-existing-image{margin-inline-start:1rem}
