# Import block types defined in submodules into the wagtail.blocks namespace
from wagtail.blocks.base import *  # NOQA
from wagtail.blocks.field_block import *  # NOQA
from wagtail.blocks.list_block import *  # NOQA
from wagtail.blocks.static_block import *  # NOQA
from wagtail.blocks.stream_block import *  # NOQA
from wagtail.blocks.struct_block import *  # NOQA
