{% extends "wagtailadmin/generic/base.html" %}
{% load i18n %}

{% block main_content %}
    <p>{{ view.confirmation_message }}</p>
    <form action="{{ view.get_delete_url }}" method="POST">
        {% csrf_token %}
        <input type="submit" value="{% trans 'Yes, delete' %}" class="button serious" />
    </form>
{% endblock %}
