!function(e){e.fn.prepopulate=function(n,a,t){return this.each((function(){var d=e(this);d.data("_changed",!1),d.on("change",(function(){d.data("_changed",!0)})),d.val()||e(n.join(",")).on("keyup change focus",(function(){if(!d.data("_changed")){var o=[];e.each(n,(function(n,a){(a=e(a)).val().length>0&&o.push(a.val())})),d.val(URLify(o.join(" "),a,t))}}))}))}}(jQuery),function(e){e((function(){var n=e("#modeladmin-prepopulated-fields-constants").data("prepopulatedFields");e.each(n,(function(n,a){e(".empty-form .form-row .field-"+a.name+", .empty-form.form-row .field-"+a.name).addClass("prepopulated_field"),e(a.id).data("dependency_list",a.dependency_list).prepopulate(a.dependency_ids,a.maxLength,a.allowUnicode)}))}))}(jQuery);