# Generated by Django 2.2.10 on 2020-07-13 10:13

from django.db import migrations

from wagtail.models import BootstrapTranslatableModel


class Migration(migrations.Migration):

    dependencies = [
        ("wagtailcore", "0055_page_locale_fields"),
    ]

    operations = [
        BootstrapTranslatableModel("wagtailcore.Page"),
    ]
