(()=>{"use strict";var e,t={3658:function(e,t,r){var n=this&&this.__importDefault||function(e){return e&&e.__esModule?e:{default:e}};t.__esModule=!0;var o=n(r(5311));window.buildExpandingFormset=function(e,t){void 0===t&&(t={});var r=(0,o.default)("#"+e+"-ADD"),n=(0,o.default)("#"+e+"-FORMS"),i=(0,o.default)("#"+e+"-TOTAL_FORMS"),a=parseInt(i.val(),10);if(t.onInit)for(var l=0;l<a;l++)t.onInit(l);var u=document.getElementById(e+"-EMPTY_FORM_TEMPLATE");u.innerText?u=u.innerText:u.textContent&&(u=u.textContent),r.on("click",(function(){if(r.hasClass("disabled"))return!1;var e=u.replace(/__prefix__(.*?['"])/g,a+"$1").replace(/<-(-*)\/script>/g,"<$1/script>");n.append(e),t.onAdd&&t.onAdd(a),t.onInit&&t.onInit(a),a++,i.val(a)}))}},5311:e=>{e.exports=jQuery}},r={};function n(e){var o=r[e];if(void 0!==o)return o.exports;var i=r[e]={exports:{}};return t[e].call(i.exports,i,i.exports,n),i.exports}n.m=t,e=[],n.O=(t,r,o,i)=>{if(!r){var a=1/0;for(f=0;f<e.length;f++){for(var[r,o,i]=e[f],l=!0,u=0;u<r.length;u++)(!1&i||a>=i)&&Object.keys(n.O).every((e=>n.O[e](r[u])))?r.splice(u--,1):(l=!1,i<a&&(a=i));if(l){e.splice(f--,1);var d=o();void 0!==d&&(t=d)}}return t}i=i||0;for(var f=e.length;f>0&&e[f-1][2]>i;f--)e[f]=e[f-1];e[f]=[r,o,i]},n.n=e=>{var t=e&&e.__esModule?()=>e.default:()=>e;return n.d(t,{a:t}),t},n.d=(e,t)=>{for(var r in t)n.o(t,r)&&!n.o(e,r)&&Object.defineProperty(e,r,{enumerable:!0,get:t[r]})},n.g=function(){if("object"==typeof globalThis)return globalThis;try{return this||new Function("return this")()}catch(e){if("object"==typeof window)return window}}(),n.o=(e,t)=>Object.prototype.hasOwnProperty.call(e,t),n.r=e=>{"undefined"!=typeof Symbol&&Symbol.toStringTag&&Object.defineProperty(e,Symbol.toStringTag,{value:"Module"}),Object.defineProperty(e,"__esModule",{value:!0})},n.j=9399,(()=>{var e={9399:0};n.O.j=t=>0===e[t];var t=(t,r)=>{var o,i,[a,l,u]=r,d=0;if(a.some((t=>0!==e[t]))){for(o in l)n.o(l,o)&&(n.m[o]=l[o]);if(u)var f=u(n)}for(t&&t(r);d<a.length;d++)i=a[d],n.o(e,i)&&e[i]&&e[i][0](),e[i]=0;return n.O(f)},r=globalThis.webpackChunkwagtail=globalThis.webpackChunkwagtail||[];r.forEach(t.bind(null,0)),r.push=t.bind(null,r.push.bind(r))})();var o=n.O(void 0,[2751],(()=>n(3658)));o=n.O(o)})();