# Generated by Django 2.0 on 2018-02-15 10:50

from django.db import migrations
import wagtail.blocks
import wagtail.fields
import wagtail.test.testapp.models


class Migration(migrations.Migration):

    dependencies = [
        ("tests", "0028_headcountrelatedmodelusingpk"),
    ]

    operations = [
        migrations.AlterField(
            model_name="streampage",
            name="body",
            field=wagtail.fields.StreamField(
                (
                    ("text", wagtail.blocks.CharBlock()),
                    ("rich_text", wagtail.blocks.RichTextBlock()),
                    ("image", wagtail.test.testapp.models.ExtendedImageChooserBlock()),
                    (
                        "product",
                        wagtail.blocks.StructBlock(
                            (
                                ("name", wagtail.blocks.CharBlock()),
                                ("price", wagtail.blocks.CharBlock()),
                            )
                        ),
                    ),
                )
            ),
        ),
    ]
