{% load i18n %}
{% get_current_language as LANGUAGE_CODE %}
{% get_current_language_bidi as LANGUAGE_BIDI %}
<!DOCTYPE HTML>
<html lang="{{ LANGUAGE_CODE }}" dir="{% if LANGUAGE_BIDI %}rtl{% else %}ltr{% endif %}">
    <head>
        <title>{% trans "Password required" %}</title>
    </head>
    <body>
        <h1>{% trans "Password required" %}</h1>
        {% block password_required_message %}
            <p>{% trans "You need a password to access this page." %}</p>
        {% endblock %}
        <form action="{{ action_url }}" method="POST">
            {% csrf_token %}
            {{ form.as_p }}
            <input type="submit" value="{% trans 'Continue' %}" class="button" />
        </form>
    </body>
</html>
