{% extends "wagtailadmin/userbar/item_base.html" %}
{% load i18n wagtailadmin_tags %}

{% block item_content %}
    <a href="{% url 'wagtailadmin_home' %}" target="_parent" class="wagtail-userbar-link" role="menuitem">
        {% icon name="wagtail-icon" class_name="wagtail-action-icon" %}
        {% trans 'Go to Wagtail admin' %}
    </a>
{% endblock %}
