(()=>{"use strict";var t={4923:t=>{function r(t,o,i){return r=n()?Reflect.construct:function(t,r,n){var o=[null];o.push.apply(o,r);var i=new(Function.bind.apply(t,o));return n&&e(i,n.prototype),i},r.apply(null,arguments)}function n(){if("undefined"==typeof Reflect||!Reflect.construct)return!1;if(Reflect.construct.sham)return!1;if("function"==typeof Proxy)return!0;try{return Date.prototype.toString.call(Reflect.construct(Date,[],(function(){}))),!0}catch(t){return!1}}function e(t,r){return e=Object.setPrototypeOf||function(t,r){return t.__proto__=r,t},e(t,r)}function o(t,r){return function(t){if(Array.isArray(t))return t}(t)||function(t,r){if("undefined"!=typeof Symbol&&Symbol.iterator in Object(t)){var n=[],e=!0,o=!1,i=void 0;try{for(var a,u=t[Symbol.iterator]();!(e=(a=u.next()).done)&&(n.push(a.value),!r||n.length!==r);e=!0);}catch(t){o=!0,i=t}finally{try{e||null==u.return||u.return()}finally{if(o)throw i}}return n}}(t,r)||i(t,r)||function(){throw new TypeError("Invalid attempt to destructure non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method.")}()}function i(t,r){if(t){if("string"==typeof t)return a(t,r);var n=Object.prototype.toString.call(t).slice(8,-1);return"Object"===n&&t.constructor&&(n=t.constructor.name),"Map"===n||"Set"===n?Array.from(t):"Arguments"===n||/^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)?a(t,r):void 0}}function a(t,r){(null==r||r>t.length)&&(r=t.length);for(var n=0,e=new Array(r);n<r;n++)e[n]=t[n];return e}function u(t){return u="function"==typeof Symbol&&"symbol"==typeof Symbol.iterator?function(t){return typeof t}:function(t){return t&&"function"==typeof Symbol&&t.constructor===Symbol&&t!==Symbol.prototype?"symbol":typeof t},u(t)}function c(t,r){for(var n=0;n<r.length;n++){var e=r[n];e.enumerable=e.enumerable||!1,e.configurable=!0,"value"in e&&(e.writable=!0),Object.defineProperty(t,e.key,e)}}var f=function(){function t(){!function(t,r){if(!(t instanceof r))throw new TypeError("Cannot call a class as a function")}(this,t),this.constructors={}}var n,e;return n=t,(e=[{key:"register",value:function(t,r){this.constructors[t]=r}},{key:"unpack",value:function(t){var r={};return this.scanForIds(t,r),this.unpackWithRefs(t,r,{})}},{key:"scanForIds",value:function(t,r){var n=this;if(null!==t&&"object"===u(t))if(Array.isArray(t))t.forEach((function(t){return n.scanForIds(t,r)}));else{var e=!1;if("_id"in t&&(e=!0,r[t._id]=t),("_type"in t||"_val"in t||"_ref"in t)&&(e=!0),"_list"in t&&(e=!0,t._list.forEach((function(t){return n.scanForIds(t,r)}))),"_args"in t&&(e=!0,t._args.forEach((function(t){return n.scanForIds(t,r)}))),"_dict"in t){e=!0;for(var i=0,a=Object.entries(t._dict);i<a.length;i++){var c=o(a[i],2),f=(c[0],c[1]);this.scanForIds(f,r)}}if(!e)for(var s=0,l=Object.entries(t);s<l.length;s++){var p=o(l[s],2),y=(p[0],p[1]);this.scanForIds(y,r)}}}},{key:"unpackWithRefs",value:function(t,n,e){var c,f,s=this;if(null===t||"object"!==u(t))return t;if(Array.isArray(t))return t.map((function(t){return s.unpackWithRefs(t,n,e)}));if("_ref"in t)c=t._ref in e?e[t._ref]:this.unpackWithRefs(n[t._ref],n,e);else if("_val"in t)c=t._val;else if("_list"in t)c=t._list.map((function(t){return s.unpackWithRefs(t,n,e)}));else if("_dict"in t){c={};for(var l=0,p=Object.entries(t._dict);l<p.length;l++){var y=o(p[l],2),h=y[0],d=y[1];c[h]=this.unpackWithRefs(d,n,e)}}else{if(!("_type"in t)){if("_id"in t)throw new Error("telepath encountered object with _id but no type specified");c={};for(var v=0,b=Object.entries(t);v<b.length;v++){var _=o(b[v],2),m=_[0],w=_[1];c[m]=this.unpackWithRefs(w,n,e)}return c}var g=t._type;c=r(this.constructors[g],function(t){if(Array.isArray(t))return a(t)}(f=t._args.map((function(t){return s.unpackWithRefs(t,n,e)})))||function(t){if("undefined"!=typeof Symbol&&Symbol.iterator in Object(t))return Array.from(t)}(f)||i(f)||function(){throw new TypeError("Invalid attempt to spread non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method.")}())}return"_id"in t&&(e[t._id]=c),c}}])&&c(n.prototype,e),t}();t.exports=f},1249:function(t,r,n){var e=this&&this.__importDefault||function(t){return t&&t.__esModule?t:{default:t}};r.__esModule=!0;var o=e(n(4923));window.telepath||(window.telepath=new o.default)}},r={};!function n(e){var o=r[e];if(void 0!==o)return o.exports;var i=r[e]={exports:{}};return t[e].call(i.exports,i,i.exports,n),i.exports}(1249)})();