.listing-with-x-scroll {
    display: block;
    overflow-x: auto;
    white-space: nowrap;
}

header .has-multiple-actions {
    display: flex;
}

header .has-multiple-actions .actionbutton {
    margin-inline-start: 10px;
}

header .has-multiple-actions .dropdown {
    margin-inline-start: 10px;
}
