# Generated by Django 3.0.8 on 2020-07-31 12:18

from django.db import migrations, models
import django.db.models.deletion
import uuid


class Migration(migrations.Migration):

    dependencies = [
        ("wagtailcore", "0057_page_locale_fields_notnull"),
        ("tests", "0054_simpletask"),
    ]

    operations = [
        migrations.AddField(
            model_name="eventcategory",
            name="locale",
            field=models.ForeignKey(
                default=1,
                editable=False,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="wagtailcore.Locale",
            ),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name="eventcategory",
            name="translation_key",
            field=models.UUIDField(default=uuid.uuid4, editable=False),
        ),
        migrations.AddField(
            model_name="eventpagecarouselitem",
            name="locale",
            field=models.ForeignKey(
                default=1,
                editable=False,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="wagtailcore.Locale",
            ),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name="eventpagecarouselitem",
            name="translation_key",
            field=models.UUIDField(default=uuid.uuid4, editable=False),
        ),
        migrations.AddField(
            model_name="eventpagerelatedlink",
            name="locale",
            field=models.ForeignKey(
                default=1,
                editable=False,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="wagtailcore.Locale",
            ),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name="eventpagerelatedlink",
            name="translation_key",
            field=models.UUIDField(default=uuid.uuid4, editable=False),
        ),
        migrations.AddField(
            model_name="eventpagespeaker",
            name="locale",
            field=models.ForeignKey(
                default=1,
                editable=False,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="wagtailcore.Locale",
            ),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name="eventpagespeaker",
            name="translation_key",
            field=models.UUIDField(default=uuid.uuid4, editable=False),
        ),
        migrations.AddField(
            model_name="eventpagespeakeraward",
            name="locale",
            field=models.ForeignKey(
                default=1,
                editable=False,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="wagtailcore.Locale",
            ),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name="eventpagespeakeraward",
            name="translation_key",
            field=models.UUIDField(default=uuid.uuid4, editable=False),
        ),
        migrations.AlterUniqueTogether(
            name="eventcategory",
            unique_together={("translation_key", "locale")},
        ),
        migrations.AlterUniqueTogether(
            name="eventpagecarouselitem",
            unique_together={("translation_key", "locale")},
        ),
        migrations.AlterUniqueTogether(
            name="eventpagerelatedlink",
            unique_together={("translation_key", "locale")},
        ),
        migrations.AlterUniqueTogether(
            name="eventpagespeaker",
            unique_together={("translation_key", "locale")},
        ),
        migrations.AlterUniqueTogether(
            name="eventpagespeakeraward",
            unique_together={("translation_key", "locale")},
        ),
    ]
