(()=>{"use strict";var e,t={4804:function(e,t,a){var r=this&&this.__importDefault||function(e){return e&&e.__esModule?e:{default:e}};t.__esModule=!0;var l=r(a(5311));(0,l.default)((function(){(0,l.default)('[data-widget="filtered-select"]').each((function(){var e=(0,l.default)("#"+this.dataset.filterField),t=(0,l.default)(this),a=[];function r(){var r=t.val();t.empty();var i,o=e.val();if(""===o)i=a;else{i=[];for(var n=0;n<a.length;n++)""!==a[n].value&&-1===a[n].filterValue.indexOf(o)||i.push(a[n])}var u=!1;for(n=0;n<i.length;n++){var f=(0,l.default)("<option>");f.attr("value",i[n].value),i[n].value===r&&(u=!0),f.text(i[n].label),t.append(f)}u?t.val(r):t.val("")}(0,l.default)("option",this).each((function(){var e;e="filterValue"in this.dataset?this.dataset.filterValue.split(","):[],a.push({value:this.value,label:this.label,filterValue:e})})),r(),e.change(r)}))}))},5311:e=>{e.exports=jQuery}},a={};function r(e){var l=a[e];if(void 0!==l)return l.exports;var i=a[e]={exports:{}};return t[e].call(i.exports,i,i.exports,r),i.exports}r.m=t,e=[],r.O=(t,a,l,i)=>{if(!a){var o=1/0;for(s=0;s<e.length;s++){for(var[a,l,i]=e[s],n=!0,u=0;u<a.length;u++)(!1&i||o>=i)&&Object.keys(r.O).every((e=>r.O[e](a[u])))?a.splice(u--,1):(n=!1,i<o&&(o=i));if(n){e.splice(s--,1);var f=l();void 0!==f&&(t=f)}}return t}i=i||0;for(var s=e.length;s>0&&e[s-1][2]>i;s--)e[s]=e[s-1];e[s]=[a,l,i]},r.n=e=>{var t=e&&e.__esModule?()=>e.default:()=>e;return r.d(t,{a:t}),t},r.d=(e,t)=>{for(var a in t)r.o(t,a)&&!r.o(e,a)&&Object.defineProperty(e,a,{enumerable:!0,get:t[a]})},r.g=function(){if("object"==typeof globalThis)return globalThis;try{return this||new Function("return this")()}catch(e){if("object"==typeof window)return window}}(),r.o=(e,t)=>Object.prototype.hasOwnProperty.call(e,t),r.r=e=>{"undefined"!=typeof Symbol&&Symbol.toStringTag&&Object.defineProperty(e,Symbol.toStringTag,{value:"Module"}),Object.defineProperty(e,"__esModule",{value:!0})},r.j=9004,(()=>{var e={9004:0};r.O.j=t=>0===e[t];var t=(t,a)=>{var l,i,[o,n,u]=a,f=0;if(o.some((t=>0!==e[t]))){for(l in n)r.o(n,l)&&(r.m[l]=n[l]);if(u)var s=u(r)}for(t&&t(a);f<o.length;f++)i=o[f],r.o(e,i)&&e[i]&&e[i][0](),e[i]=0;return r.O(s)},a=globalThis.webpackChunkwagtail=globalThis.webpackChunkwagtail||[];a.forEach(t.bind(null,0)),a.push=t.bind(null,a.push.bind(a))})();var l=r.O(void 0,[2751],(()=>r(4804)));l=r.O(l)})();