{% load i18n %}
<td>
    {% include "wagtailadmin/shared/field.html" with field=form.page only %}
    {{ form.DELETE }}
</td>

<td>
    <button class="button button-secondary button-small no" type="button" id="{{ form.DELETE.id_for_label }}-button">{% trans "Delete" %}</button>
</td>
