# Generated by Django 2.2.6 on 2019-10-16 08:30

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ("wagtailcore", "0042_index_on_pagerevision_approved_go_live_at"),
    ]

    operations = [
        migrations.AddField(
            model_name="page",
            name="locked_at",
            field=models.DateTimeField(
                editable=False, null=True, verbose_name="locked at"
            ),
        ),
        migrations.AddField(
            model_name="page",
            name="locked_by",
            field=models.ForeignKey(
                blank=True,
                editable=False,
                null=True,
                on_delete=django.db.models.deletion.SET_NULL,
                related_name="locked_pages",
                to=settings.AUTH_USER_MODEL,
                verbose_name="locked by",
            ),
        ),
    ]
