# Generated by Django 2.1.2 on 2018-10-24 09:54

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("tests", "0038_singletonpageviamaxcount"),
    ]

    operations = [
        migrations.AddField(
            model_name="customdocument",
            name="description",
            field=models.TextField(blank=True),
        ),
    ]
