.workflow-progress-tabs__buttons{margin-top:30px}.workflow-progress-tabs__button{background:none;border:0;font-weight:700;padding-bottom:5px}.workflow-progress-tabs__button--active{border-bottom:3px solid #007d7e}.workflow-progress-tabs__tab{display:none}.workflow-progress-tabs__tab--active{display:block}.workflow-progress-table{border-bottom:1px solid #e5e5e5;border-top:1px solid #e5e5e5;font-size:.8em;width:100%}.workflow-progress-table td,.workflow-progress-table th{padding:20px}.workflow-progress-table th{font-size:1.2em}.workflow-progress-table__left-column{color:#555;font-size:1.2em;font-weight:700}.workflow-progress-table--tasks{background-color:#fafafa}.workflow-progress-table--tasks .workflow-progress-table__left-column{background-color:#f5f5f5}.workflow-progress-table--timeline tr{border-bottom:1px solid #e5e5e5}
