# -*- coding: utf-8 -*-
# Generated by Django 1.11.6 on 2017-10-13 16:10
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("tests", "0020_customdocument"),
    ]

    operations = [
        migrations.AlterField(
            model_name="formfield",
            name="field_type",
            field=models.CharField(
                choices=[
                    ("singleline", "Single line text"),
                    ("multiline", "Multi-line text"),
                    ("email", "Email"),
                    ("number", "Number"),
                    ("url", "URL"),
                    ("checkbox", "Checkbox"),
                    ("checkboxes", "Checkboxes"),
                    ("dropdown", "Drop down"),
                    ("multiselect", "Multiple select"),
                    ("radio", "Radio buttons"),
                    ("date", "Date"),
                    ("datetime", "Date/time"),
                    ("hidden", "Hidden field"),
                ],
                max_length=16,
                verbose_name="field type",
            ),
        ),
        migrations.AlterField(
            model_name="formfieldwithcustomsubmission",
            name="field_type",
            field=models.CharField(
                choices=[
                    ("singleline", "Single line text"),
                    ("multiline", "Multi-line text"),
                    ("email", "Email"),
                    ("number", "Number"),
                    ("url", "URL"),
                    ("checkbox", "Checkbox"),
                    ("checkboxes", "Checkboxes"),
                    ("dropdown", "Drop down"),
                    ("multiselect", "Multiple select"),
                    ("radio", "Radio buttons"),
                    ("date", "Date"),
                    ("datetime", "Date/time"),
                    ("hidden", "Hidden field"),
                ],
                max_length=16,
                verbose_name="field type",
            ),
        ),
        migrations.AlterField(
            model_name="jadeformfield",
            name="field_type",
            field=models.CharField(
                choices=[
                    ("singleline", "Single line text"),
                    ("multiline", "Multi-line text"),
                    ("email", "Email"),
                    ("number", "Number"),
                    ("url", "URL"),
                    ("checkbox", "Checkbox"),
                    ("checkboxes", "Checkboxes"),
                    ("dropdown", "Drop down"),
                    ("multiselect", "Multiple select"),
                    ("radio", "Radio buttons"),
                    ("date", "Date"),
                    ("datetime", "Date/time"),
                    ("hidden", "Hidden field"),
                ],
                max_length=16,
                verbose_name="field type",
            ),
        ),
    ]
