{% extends "wagtailadmin/base.html" %}
{% load i18n %}
{% block titletag %}{{ page_title }}{% endblock %}
{% block extra_js %}
    {{ block.super }}
    <script>
        window.headerSearch = {
            url: "{% url view.index_url_name %}",
            termInput: "#id_q",
            targetOutput: "#group-results"
        }
    </script>
{% endblock %}

{% block content %}
    {% url view.add_url_name as add_link %}
    {% include "wagtailadmin/shared/header.html" with title=page_title action_url=add_link action_text=view.add_item_label icon=header_icon search_url=view.index_url_name %}

    <div class="nice-padding">
        <div id="group-results" class="groups">
            {% include "wagtailusers/groups/results.html" %}
        </div>
    </div>
{% endblock %}
