(()=>{"use strict";var o,t={3152:function(o,t,e){var n=this&&this.__importDefault||function(o){return o&&o.__esModule?o:{default:o}};t.__esModule=!0;var a=n(e(5311));function r(o,t,e){var n=document.createElement("input");n.type="hidden",n.name=t,n.value=e,o.appendChild(n)}window._addHiddenInput=r,window.ActivateWorkflowActionsForDashboard=function(o){document.querySelectorAll("[data-workflow-action-url]").forEach((function(t){t.addEventListener("click",(function(e){if(e.preventDefault(),e.stopPropagation(),"launchModal"in t.dataset)ModalWorkflow({url:t.dataset.workflowActionUrl,onload:{action:function(o){var t=document.createElement("input");t.type="hidden",t.name="next",t.value=window.location,(0,a.default)("form",o.body).append(t),o.ajaxifyForm((0,a.default)("form",o.body))},success:function(o,t){window.location.href=t.redirect}}});else{var n=document.createElement("form");n.action=t.dataset.workflowActionUrl,n.method="POST",r(n,"csrfmiddlewaretoken",o),r(n,"next",window.location),document.body.appendChild(n),n.submit()}}),{capture:!0})}))},window.ActivateWorkflowActionsForEditView=function(o){var t=(0,a.default)(o).get(0);document.querySelectorAll("[data-workflow-action-name]").forEach((function(o){o.addEventListener("click",(function(e){"workflowActionModalUrl"in o.dataset?(e.preventDefault(),e.stopPropagation(),ModalWorkflow({url:o.dataset.workflowActionModalUrl,onload:{action:function(o){o.ajaxifyForm((0,a.default)("form",o.body))},success:function(e,n){r(t,"action-workflow-action","true"),r(t,"workflow-action-name",o.dataset.workflowActionName),r(t,"workflow-action-extra-data",JSON.stringify(n.cleaned_data)),(0,a.default)(t).submit()}}})):(r(t,"action-workflow-action","true"),r(t,"workflow-action-name",o.dataset.workflowActionName))}),{capture:!0})}))}},5311:o=>{o.exports=jQuery}},e={};function n(o){var a=e[o];if(void 0!==a)return a.exports;var r=e[o]={exports:{}};return t[o].call(r.exports,r,r.exports,n),r.exports}n.m=t,o=[],n.O=(t,e,a,r)=>{if(!e){var i=1/0;for(u=0;u<o.length;u++){for(var[e,a,r]=o[u],l=!0,c=0;c<e.length;c++)(!1&r||i>=r)&&Object.keys(n.O).every((o=>n.O[o](e[c])))?e.splice(c--,1):(l=!1,r<i&&(i=r));if(l){o.splice(u--,1);var d=a();void 0!==d&&(t=d)}}return t}r=r||0;for(var u=o.length;u>0&&o[u-1][2]>r;u--)o[u]=o[u-1];o[u]=[e,a,r]},n.n=o=>{var t=o&&o.__esModule?()=>o.default:()=>o;return n.d(t,{a:t}),t},n.d=(o,t)=>{for(var e in t)n.o(t,e)&&!n.o(o,e)&&Object.defineProperty(o,e,{enumerable:!0,get:t[e]})},n.g=function(){if("object"==typeof globalThis)return globalThis;try{return this||new Function("return this")()}catch(o){if("object"==typeof window)return window}}(),n.o=(o,t)=>Object.prototype.hasOwnProperty.call(o,t),n.r=o=>{"undefined"!=typeof Symbol&&Symbol.toStringTag&&Object.defineProperty(o,Symbol.toStringTag,{value:"Module"}),Object.defineProperty(o,"__esModule",{value:!0})},n.j=5477,(()=>{var o={5477:0};n.O.j=t=>0===o[t];var t=(t,e)=>{var a,r,[i,l,c]=e,d=0;if(i.some((t=>0!==o[t]))){for(a in l)n.o(l,a)&&(n.m[a]=l[a]);if(c)var u=c(n)}for(t&&t(e);d<i.length;d++)r=i[d],n.o(o,r)&&o[r]&&o[r][0](),o[r]=0;return n.O(u)},e=globalThis.webpackChunkwagtail=globalThis.webpackChunkwagtail||[];e.forEach(t.bind(null,0)),e.push=t.bind(null,e.push.bind(e))})();var a=n.O(void 0,[2751],(()=>n(3152)));a=n.O(a)})();