# -*- coding: utf-8 -*-
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("customuser", "0001_initial"),
    ]

    operations = [
        migrations.AddField(
            model_name="CustomUser",
            name="attachment",
            field=models.FileField(blank=True),
        ),
    ]
