{% load wagtailadmin_tags i18n %}
<!-- Wagtail user bar embed code -->
<div class="wagtail-userbar-reset">
    <div class="wagtail-userbar wagtail-userbar--{{ position|default:'bottom-right' }}" data-wagtail-userbar>
        <link rel="stylesheet" href="{% versioned_static 'wagtailadmin/css/userbar.css' %}" type="text/css" />
        <div class="wagtail-userbar-nav">
            <button aria-controls="wagtail-userbar-items" aria-haspopup="true" class="wagtail-userbar-trigger" id="wagtail-userbar-trigger" data-wagtail-userbar-trigger>
                {% block branding_logo %}
                    <div style="display: none">
                        <svg>
                            <defs>
                                {% include "wagtailadmin/icons/wagtail.svg" %}
                                {% include "wagtailadmin/icons/folder-open-inverse.svg" %}
                                {% include "wagtailadmin/icons/edit.svg" %}
                                {% include "wagtailadmin/icons/plus.svg" %}
                                {% include "wagtailadmin/icons/tick.svg" %}
                                {% include "wagtailadmin/icons/cross.svg" %}
                            </defs>
                        </svg>
                    </div>
                    <svg class="wagtail-userbar-icon" aria-hidden="true">
                        <use href="#icon-wagtail-icon"></use>
                    </svg>
                {% endblock %}
                <span class="wagtail-userbar-help-text">{% trans 'View Wagtail quick actions' %}</span>
            </button>
            <ul aria-labelledby="wagtail-userbar-trigger" class="wagtail-userbar-items" id="wagtail-userbar-items" role="menu">
                {% for item in items %}
                    {{ item|safe }}
                {% endfor %}
            </ul>
        </div>
        <script src="{% versioned_static 'wagtailadmin/js/userbar.js' %}"></script>
    </div>
</div>
<!-- end Wagtail user bar embed code -->
