{% extends "wagtailadmin/generic/base.html" %}
{% load i18n %}

{% block main_content %}
    {% block before_form %}{% endblock %}
    <form action="{{ action_url }}" method="POST" novalidate{% if form.is_multipart %} enctype="multipart/form-data"{% endif %}>
        {% csrf_token %}

        {% block hidden_fields %}
            {% for field in form.hidden_fields %}{{ field }}{% endfor %}
        {% endblock %}

        <ul class="fields">
            {% block visible_fields %}
                {% for field in form.visible_fields %}
                    {% include "wagtailadmin/shared/field_as_li.html" %}
                {% endfor %}
            {% endblock %}

            <li>
                {% block actions %}
                    <input type="submit" value="{{ submit_button_label }}" class="button" />
                {% endblock %}
            </li>
        </ul>
    </form>
{% endblock %}

{% block extra_js %}
    {{ block.super }}
    {{ form.media.js }}
{% endblock %}

{% block extra_css %}
    {{ block.super }}
    {{ form.media.css }}
{% endblock %}
