# Generated by Django 2.2.5 on 2019-12-06 15:54

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ("tests", "0044_merge_20191203_0116"),
    ]

    operations = [
        migrations.AlterModelOptions(
            name="customformpagesubmission",
            options={
                "verbose_name": "form submission",
                "verbose_name_plural": "form submissions",
            },
        ),
    ]
