# Generated by Django 3.2.6 on 2022-04-05 13:24

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ("wagtailredirects", "0007_add_autocreate_fields"),
    ]

    operations = [
        migrations.AlterModelOptions(
            name="redirect",
            options={"verbose_name": "redirect", "verbose_name_plural": "redirects"},
        ),
    ]
