{% load i18n wagtailadmin_tags %}
<section class="footer bulk-actions-choices hidden" data-bulk-action-footer aria-labelledby="bulk-actions-heading">
    <h2 id="bulk-actions-heading" class="visuallyhidden">{% trans "Bulk actions" %}</h2>
    <div class="footer__container">
        {% include 'wagtailadmin/bulk_actions/select_all_checkbox_input.html' with parent=parent %}
        <ul class="bulk-actions-buttons">{% bulk_action_choices app_label model_name %}</ul>
        <span data-bulk-action-num-objects class="num-objects"></span>
        {% if objects.has_other_pages %}
            <button type="button" data-bulk-action-num-objects-in-listing class="num-objects-in-listing u-hidden">{{ select_all_obj_text }}</button>
        {% endif %}
    </div>
</section>
