.top-padding{padding-top:52px}.avatar-panel{margin-top:20px}.avatar-panel__image{float:left;margin-bottom:20px;width:16%}.avatar-panel__form{float:left}.avatar-panel__form label{display:block;float:unset;margin-bottom:10px;padding:0;width:unset}.avatar-panel__revert-to-gravatar{margin-bottom:20px;margin-top:20px}.avatar-panel:after{clear:both;content:" ";display:table}
