{% load wagtailcore_tags %}
<table>
    <thead>
        <tr>
            {% for col in value.columns %}
                <th scope="col">{{ col.heading }}</th>
            {% endfor %}
        </tr>
    </thead>
    <tbody>
        {% for row in value.rows %}
            <tr>
                {% for block in row %}
                    <td>{% include_block block %}</td>
                {% endfor %}
            </tr>
        {% endfor %}
    </tbody>
</table>
