# Generated by Django 2.1.11 on 2019-09-06 15:11

from django.db import migrations, models
import django.db.models.deletion
import modelcluster.fields


class Migration(migrations.Migration):

    dependencies = [
        ("tests", "0042_simplechildpage_simpleparentpage"),
    ]

    operations = [
        migrations.CreateModel(
            name="EventPageSpeakerAward",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "sort_order",
                    models.IntegerField(blank=True, editable=False, null=True),
                ),
                ("name", models.CharField(max_length=255, verbose_name="Award name")),
                ("date_awarded", models.DateField(blank=True, null=True)),
                (
                    "speaker",
                    modelcluster.fields.ParentalKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="awards",
                        to="tests.EventPageSpeaker",
                    ),
                ),
            ],
            options={
                "ordering": ["sort_order"],
                "abstract": False,
            },
        ),
    ]
