# Generated by Django 2.2.5 on 2019-12-02 16:16

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ("tests", "0043_customdocument_fancy_description"),
        ("tests", "0043_eventpagespeakeraward"),
    ]

    operations = []
