# Generated by Django 2.2.dev20181026000358 on 2018-10-27 09:42

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ("wagtailcore", "0040_page_draft_title"),
    ]

    operations = [
        migrations.AlterModelOptions(
            name="groupcollectionpermission",
            options={
                "verbose_name": "group collection permission",
                "verbose_name_plural": "group collection permissions",
            },
        ),
    ]
