{% extends "wagtailadmin/base.html" %}
{% load i18n modeladmin_tags wagtailadmin_tags %}

{% block titletag %}{{ view.get_meta_title }}{% endblock %}

{% block extra_css %}
    {{ block.super }}
    {% include "wagtailadmin/pages/_editor_css.html" %}
    {{ form.media.css }}
    <link rel="stylesheet" href="{% versioned_static 'wagtailmodeladmin/css/choose_parent_page.css' %}" type="text/css"/>
    <link rel="stylesheet" href="{% versioned_static 'wagtailmodeladmin/css/breadcrumbs_page.css' %}" type="text/css"/>
{% endblock %}

{% block extra_js %}
    {{ block.super }}
    {% include "wagtailadmin/pages/_editor_js.html" %}
    {{ form.media.js }}
{% endblock %}


{% block content %}

    {% block header %}
        {% include "modeladmin/includes/header_with_breadcrumb.html" with title=view.get_page_title subtitle=view.get_page_subtitle icon=view.header_icon %}
    {% endblock %}

    <div>

        <div class="nice-padding">
            <h2>{% blocktrans trimmed %}Choose a parent page{% endblocktrans %}</h2>
            <p>{% blocktrans trimmed with view.verbose_name_plural|capfirst as plural %}{{ plural }} can be added to more than one place within your site. Which of the following would you like to be the parent of your new page?{% endblocktrans %}</p>

            <form action="" method="post" novalidate>
                {% csrf_token %}

                <ul class="fields">
                    {% include "wagtailadmin/shared/field_as_li.html" with field=form.parent_page %}
                    <li>
                        <input type="submit" class="button" value="{% trans 'Continue' %}">
                    </li>
                </ul>
            </form>

        </div>
    </div>
{% endblock %}
