.replace-file-input{display:inline-block;overflow:hidden;padding-bottom:2px;position:relative}.replace-file-input [type=file]{direction:ltr;display:block;font-size:5em;inset-inline-end:0;opacity:0;padding:0;position:absolute;right:0;top:0;width:auto}.replace-file-input [type=file]:hover,.replace-file-input:hover{cursor:pointer}.replace-file-input:hover button{background-color:var(--color-primary-darker)}.upload-list>li{padding:1em}.upload-list .left{text-align:center;word-break:break-all}.upload-list .preview{display:block;margin:auto;max-width:100%;min-height:150px;position:relative;text-align:center;width:150px}.upload-list .progress,.upload-list .thumb,.upload-list .thumb:before,.upload-list canvas,.upload-list img{max-width:100%;position:absolute}.upload-list .progress{box-shadow:0 0 5px 2px #fff6;inset-inline-end:20%;inset-inline-start:20%;left:20%;right:20%;top:60%;width:60%;z-index:4}.upload-list .thumb{bottom:0;inset-inline-end:0;inset-inline-start:0;left:0;right:0;top:0;width:100%;z-index:1}.upload-list .thumb:before,.upload-list canvas,.upload-list img{bottom:0;inset-inline-end:0;inset-inline-start:0;left:0;margin:auto;right:0;top:0}.upload-list .thumb:before{color:#f0f0f0;font-size:10em;line-height:1.4em;top:0;width:1em;z-index:2}.upload-list canvas,.upload-list img{z-index:3}.upload-list .hasthumb:before,.upload-list .status-msg{display:none}.upload-list .upload-complete .progress{opacity:0}.upload-list .upload-success .status-msg.success{display:block}.upload-list .upload-duplicate{align-items:center;display:flex}.upload-list .upload-duplicate .status-msg.warning{display:block}.upload-list .upload-failure{border-color:#cd3238}.upload-list .upload-failure .preview{display:none}.upload-list .upload-failure .status-msg.failure{display:block}.upload-list .upload-server-error{border-color:#cd3238}.upload-list .upload-server-error .preview{display:none}.upload-list .upload-server-error .status-msg.server-error{display:block}.confirm-duplicate-upload,.confirm-duplicate-upload figure{align-items:center;display:flex;flex-direction:column}.confirm-duplicate-upload figure{margin:1rem auto}.confirm-duplicate-upload figure img{position:relative}.confirm-duplicate-upload form{display:inline}
