# -*- coding: utf-8 -*-
# Generated by Django 1.11.6 on 2018-01-10 00:25
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion
import modelcluster.fields


class Migration(migrations.Migration):

    dependencies = [
        ("tests", "0027_auto_20180110_1727"),
    ]

    operations = [
        migrations.CreateModel(
            name="HeadCountRelatedModelUsingPK",
            fields=[
                ("custom_id", models.AutoField(primary_key=True, serialize=False)),
                ("head_count", models.IntegerField()),
                (
                    "event_page",
                    modelcluster.fields.ParentalKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="head_counts",
                        to="tests.EventPage",
                    ),
                ),
            ],
        ),
    ]
