{% load wagtailadmin_tags %}
<script src="{% versioned_static 'wagtailadmin/js/expanding-formset.js' %}"></script>
<script src="{% versioned_static 'wagtailadmin/js/modal-workflow.js' %}"></script>
<script src="{% versioned_static 'wagtailusers/js/group-form.js' %}"></script>

{{ form_media.js }}
