.focal-point-chooser{margin-bottom:20px;position:relative}.focal-point-chooser .current-focal-point-indicator{border:3px solid var(--color-primary);box-shadow:1px 1px 10px 0 #000;opacity:.7;position:absolute}body.ready .focal-point-chooser .current-focal-point-indicator{transition:opacity .2s ease}.focal-point-chooser .current-focal-point-indicator.hidden{display:none}.focal-point-chooser:hover .current-focal-point-indicator{opacity:0}
