{% extends "wagtailadmin/base.html" %}
{% load i18n wagtailadmin_tags %}
{% block titletag %}{% blocktrans trimmed with title=page.get_admin_display_title %}Delete {{ title }}{% endblocktrans %}{% endblock %}

{% block content %}
    {% trans "Delete" as del_str %}
    {% include "wagtailadmin/shared/header.html" with title=del_str subtitle=page.get_admin_display_title icon="doc-empty-inverse" %}

    <div class="nice-padding">
        <p>
            {% trans 'Are you sure you want to delete this page?' %}
            {% if descendant_count %}
                {% blocktrans trimmed count counter=descendant_count %}
                    This will also delete one more subpage.
                {% plural %}
                    This will also delete {{ descendant_count }} more subpages.
                {% endblocktrans %}
            {% endif %}
        </p>

        <form action="{% url 'wagtailadmin_pages:delete' page.id %}" method="POST">
            {% csrf_token %}
            <input type="hidden" name="next" value="{{ next }}">
            <input type="submit" value="{% trans 'Yes, delete it' %}" class="button serious">
            <a href="{% if next %}{{ next }}{% else %}{% url 'wagtailadmin_explore' page.get_parent.id %}{% endif %}" class="button button-secondary">{% trans "No, don't delete it" %}</a>
        </form>

        {% page_permissions page as page_perms %}
        {% if page_perms.can_unpublish %}
            {% url 'wagtailadmin_pages:unpublish' page.id as unpublish_url %}
            <p style="margin-top: 1em">{% blocktrans trimmed %}Alternatively you can <a href="{{ unpublish_url }}">unpublish the page</a>. This removes the page from public view and you can edit or publish it again later.{% endblocktrans %}</p>
        {% endif %}
    </div>
{% endblock %}
