# -*- coding: utf-8 -*-
# Generated by Django 1.10 on 2016-08-11 12:03
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ("wagtailimages", "0013_make_rendition_upload_callable"),
    ]

    operations = [
        migrations.AddField(
            model_name="rendition",
            name="filter_spec",
            field=models.CharField(
                blank=True, db_index=True, default="", max_length=255
            ),
        ),
        migrations.AlterField(
            model_name="rendition",
            name="filter",
            field=models.ForeignKey(
                blank=True,
                null=True,
                on_delete=django.db.models.deletion.CASCADE,
                related_name="+",
                to="wagtailimages.Filter",
            ),
        ),
    ]
