(()=>{"use strict";var e,t={3023:function(e,t,o){var r=this&&this.__importDefault||function(e){return e&&e.__esModule?e:{default:e}};t.__esModule=!0;var n=r(o(5311));function l(){(0,n.default)(".object.collapsible").each((function(){var e=(0,n.default)(this),t=e.find(".object-layout"),o=function(){return t.get(0).dispatchEvent(new CustomEvent("commentAnchorVisibilityChange",{bubbles:!0}))};e.hasClass("collapsed")&&0===e.find(".error-message").length&&t.hide({complete:o}),e.find("> .title-wrapper").on("click",(function(){e.hasClass("collapsed")?(e.removeClass("collapsed"),t.show({duration:"slow",complete:o})):(e.addClass("collapsed"),t.hide({duration:"slow",complete:o}))}))}))}window.initCollapsibleBlocks=l,(0,n.default)((function(){l()}))},5311:e=>{e.exports=jQuery}},o={};function r(e){var n=o[e];if(void 0!==n)return n.exports;var l=o[e]={exports:{}};return t[e].call(l.exports,l,l.exports,r),l.exports}r.m=t,e=[],r.O=(t,o,n,l)=>{if(!o){var i=1/0;for(c=0;c<e.length;c++){for(var[o,n,l]=e[c],a=!0,s=0;s<o.length;s++)(!1&l||i>=l)&&Object.keys(r.O).every((e=>r.O[e](o[s])))?o.splice(s--,1):(a=!1,l<i&&(i=l));if(a){e.splice(c--,1);var u=n();void 0!==u&&(t=u)}}return t}l=l||0;for(var c=e.length;c>0&&e[c-1][2]>l;c--)e[c]=e[c-1];e[c]=[o,n,l]},r.n=e=>{var t=e&&e.__esModule?()=>e.default:()=>e;return r.d(t,{a:t}),t},r.d=(e,t)=>{for(var o in t)r.o(t,o)&&!r.o(e,o)&&Object.defineProperty(e,o,{enumerable:!0,get:t[o]})},r.g=function(){if("object"==typeof globalThis)return globalThis;try{return this||new Function("return this")()}catch(e){if("object"==typeof window)return window}}(),r.o=(e,t)=>Object.prototype.hasOwnProperty.call(e,t),r.r=e=>{"undefined"!=typeof Symbol&&Symbol.toStringTag&&Object.defineProperty(e,Symbol.toStringTag,{value:"Module"}),Object.defineProperty(e,"__esModule",{value:!0})},r.j=1778,(()=>{var e={1778:0};r.O.j=t=>0===e[t];var t=(t,o)=>{var n,l,[i,a,s]=o,u=0;if(i.some((t=>0!==e[t]))){for(n in a)r.o(a,n)&&(r.m[n]=a[n]);if(s)var c=s(r)}for(t&&t(o);u<i.length;u++)l=i[u],r.o(e,l)&&e[l]&&e[l][0](),e[l]=0;return r.O(c)},o=globalThis.webpackChunkwagtail=globalThis.webpackChunkwagtail||[];o.forEach(t.bind(null,0)),o.push=t.bind(null,o.push.bind(o))})();var n=r.O(void 0,[2751],(()=>r(3023)));n=r.O(n)})();