$((function(){buildExpandingFormset("id_page_permissions",{onInit:function(i){var n="id_page_permissions-"+i+"-DELETE",o="inline_child_page_permissions-"+i;$("#"+n+"-button").on("click",(function(){$("#"+n).val("1"),$("#"+o).fadeOut()}))}})}));