# -*- coding: utf-8 -*-
# Generated by Django 1.9 on 2015-12-22 09:34
from django.db import migrations, models

import wagtail.models


class Migration(migrations.Migration):

    dependencies = [
        ("wagtailcore", "0023_alter_page_revision_on_delete_behaviour"),
    ]

    operations = [
        migrations.AlterField(
            model_name="page",
            name="content_type",
            field=models.ForeignKey(
                on_delete=models.SET(wagtail.models.get_default_page_content_type),
                related_name="pages",
                to="contenttypes.ContentType",
                verbose_name="content type",
            ),
        ),
    ]
