<td class="{% if column.classname %}{{ column.classname }} {% endif %}title">
    <div class="title-wrapper">
        {% if link_url %}
            <a href="{{ link_url }}" {% if link_classname %}class="{{ link_classname }}"{% endif %}>{{ value }}</a>
        {% else %}
            {{ value }}
        {% endif %}
    </div>
</td>
