.report{grid-column-gap:50px;display:grid;grid-template-columns:auto;margin-inline-end:50px;margin-inline-start:50px}.report--has-filters{grid-template-columns:auto 300px}.report__results{grid-column-start: col-start 1 col-end 2}.report__results--text{margin:0 .5em .5em 0}.report__results--text+.status-tag{margin-inline-start:0}.report__results--comment{display:block}.report__filters{grid-column-start: col-start -2 col-end -1}.report__filters button[type=submit]{display:block;margin-bottom:20px}.report__filters input[type=checkbox]{display:block;margin-bottom:10px}.report__filters label{display:block;float:unset;padding-top:1.2em;width:unset}.report__actions>div{display:block;float:right;margin-inline-end:10px}@media screen and (max-width:56.1875em){.report--has-filters{grid-template-columns:auto}.report__filters{grid-row:1}.report form{margin-bottom:1em}}
