{% extends "wagtailadmin/generic/edit.html" %}
{% load wagtailadmin_tags %}

{% block extra_js %}
    {{ block.super }}
    <script src="{% versioned_static 'wagtailadmin/js/modal-workflow.js' %}"></script>
{% endblock %}
