h1{font-size:2.2em;font-weight:300}header{--tw-text-opacity:1;color:rgb(46 31 94/var(--tw-text-opacity));display:flex;flex-flow:row nowrap;gap:1em}header .col1{margin-inline-end:1em;padding:0;width:50px}@media screen and (max-width:49.9375em){header .col1{inset-inline-start:50px;left:50px;position:relative}}header .avatar{flex-shrink:0;margin-top:.5em}header .user-name{font-size:1.3em;font-weight:600}.panel{margin-bottom:4em}.summary{margin-bottom:2em;padding-top:2em}.summary:after,.summary:before{content:" ";display:table}.summary:after{clear:both}.summary ul.stats{font-style:normal;list-style-type:none;margin-bottom:0;margin-inline-start:-1.5%;margin-top:0;padding-inline-start:0;width:100%}.summary ul.stats:after,.summary ul.stats:before{content:" ";display:table}.summary ul.stats:after{clear:both}.summary ul.stats li{box-sizing:border-box;display:inline;float:left;font-style:normal;list-style-type:none;margin-bottom:2em;padding-inline-end:1.5%;padding-inline-start:1.5%;width:33.3333333333%}@media screen and (max-width:56.1875em){.summary ul.stats li{width:50%}}.summary ul.stats li svg.icon,.summary ul.stats li.icon:before{float:left;font-size:6em;opacity:.2}@media screen and (max-width:74.9375em){.summary ul.stats li svg.icon,.summary ul.stats li.icon:before{font-size:5em}}@media screen and (max-width:56.1875em){.summary ul.stats li svg.icon,.summary ul.stats li.icon:before{font-size:4.5em}}@media screen and (max-width:49.9375em){.summary ul.stats li svg.icon,.summary ul.stats li.icon:before{font-size:4em}}@media(forced-colors:active){.summary ul.stats li svg.icon,.summary ul.stats li.icon:before{color:LinkText;opacity:1}}.summary ul.stats li svg.icon{height:1em;margin-inline-end:.5em;width:1em}.summary ul.stats a{display:block;height:100%;position:relative;width:100%}.summary ul.stats span{display:block;font-family:-apple-system,BlinkMacSystemFont,Segoe UI,system-ui,Roboto,Helvetica Neue,Arial,sans-serif,Apple Color Emoji,Segoe UI Emoji,Segoe UI Symbol,Noto Color Emoji;font-size:4em;font-weight:300;line-height:1em}@media screen and (max-width:74.9375em){.summary ul.stats span{font-size:3em}}@media screen and (max-width:56.1875em){.summary ul.stats span{font-size:2.5em}}@media screen and (max-width:49.9375em){.summary ul.stats span{font-size:2em}}.summary .search-bar{border-top:1px solid #e6e6e6;padding:1em 0 0}.summary .search-bar:after,.summary .search-bar:before{content:" ";display:table}.summary .search-bar:after{clear:both}.summary .search-bar h2{display:inline-block;margin-inline-end:1em}.summary .search-bar .fields{clear:none;display:inline-block}.summary .search-bar .field,.summary .search-bar label{display:inline-block}.summary .search-bar input{background-color:#fff}.object.collapsible .object-layout .title-wrapper:before{display:none}.object-layout{display:flex;flex-flow:row-reverse wrap}.object-layout .listing--push-top{margin-top:3em}.object-layout .listing--push-top thead{display:table-row-group;margin-bottom:0}.object-layout .listing{margin-bottom:0}.object-layout .listing tbody{border-bottom:0}.task .icon{margin-inline-start:-1.75em}
