{% extends 'wagtailadmin/bulk_actions/confirmation/list_items_with_no_access.html' %}
{% load i18n %}

{% block per_item %}
    {% if item.can_edit %}
        <a href="{% url 'wagtailadmin_pages:edit' item.item.id %}" target="_blank" rel="noreferrer">{{ item.item.get_admin_display_title }}</a>
    {% else %}
        {{ item.item.get_admin_display_title }}
    {% endif %}
{% endblock per_item %}
