.replace-file-input{display:inline-block;overflow:hidden;padding-bottom:2px;position:relative}.replace-file-input [type=file]{direction:ltr;display:block;font-size:5em;inset-inline-end:0;opacity:0;padding:0;position:absolute;right:0;top:0;width:auto}.replace-file-input [type=file]:hover,.replace-file-input:hover{cursor:pointer}.replace-file-input:hover button{background-color:var(--color-primary-darker)}.upload-list>li{padding:1em}.upload-list .left{text-align:center;word-break:break-all}.upload-list .preview{display:block;margin:auto;max-width:100%;min-height:150px;position:relative;text-align:center;width:150px}.upload-list .progress{box-shadow:0 0 5px 2px #fff6;margin-inline-end:20%;margin-inline-start:20%;max-width:100%;width:60%;z-index:4}.upload-list .status-msg{display:none}.upload-list .upload-complete .progress{opacity:0}.upload-list .upload-success .status-msg.success{display:block}.upload-list .upload-failure{border-color:#cd3238}.upload-list .upload-failure .preview{display:none}.upload-list .upload-failure .status-msg.failure{display:block}
