# -*- coding: utf-8 -*-
# Generated by Django 1.10.4 on 2016-12-20 10:57
from django.db import migrations
import wagtail.test.testapp.models
import wagtail.blocks
import wagtail.fields


class Migration(migrations.Migration):

    dependencies = [
        ("tests", "0012_panelsettings_tabbedsettings"),
    ]

    operations = [
        migrations.AlterField(
            model_name="streampage",
            name="body",
            field=wagtail.fields.StreamField(
                [
                    ("text", wagtail.blocks.CharBlock()),
                    ("rich_text", wagtail.blocks.RichTextBlock()),
                    ("image", wagtail.test.testapp.models.ExtendedImageChooserBlock()),
                ]
            ),
        ),
    ]
