{% load wagtailadmin_tags i18n %}

<aside class="form-side" data-form-side>
    <button type="button" class="form-side__close-button" data-form-side-close-button aria-label="{% trans 'Toggle side panel' %}">
        {% icon name="expand-right" %}
    </button>

    {% for panel in side_panels %}
        <div class="form-side__panel" data-side-panel="{{ panel.name }}" hidden>
            <h2 id="side-panel-{{ panel.name }}-title" class="w-sr-only">{{ panel.title }}</h2>
            {% component panel %}
        </div>
    {% endfor %}
</aside>
