{% extends "wagtailadmin/shared/header_with_locale_selector.html" %}
{% load wagtailadmin_tags i18n %}

{% block extra_rows %}
    {% url 'wagtailsnippets:history' model_opts.app_label model_opts.model_name instance.pk|admin_urlquote as history_url %}

    {% if latest_log_entry %}
        <div class="row last-updated">
            <ul>
                <li>
                    <span class="avatar small" data-wagtail-tooltip="{{ latest_log_entry.user_display_name }}"><img src="{% avatar_url latest_log_entry.user size=25 %}" alt="" /></span>
                    {% trans "Last updated" %}
                    {% include "wagtailadmin/shared/last_updated.html" with last_updated=latest_log_entry.timestamp time_prefix="at" %}
                </li>
                {% if history_url %}
                    <li><a href="{{ history_url }}">{% icon "history" class_name="default" %} {% trans "History" %}</a></li>
                {% endif %}
                <li>{% icon "snippet" class_name="default" %} {{ model_opts.verbose_name|capfirst }}</li>
            </ul>
        </div>
    {% endif %}
{% endblock %}
