{% extends "modeladmin/create.html" %}
{% load i18n wagtailadmin_tags %}

{% block header %}
    {% include "modeladmin/includes/header_with_history.html" with title=view.get_page_title subtitle=view.get_page_subtitle icon=view.header_icon merged=1 latest_log_entry=latest_log_entry history_url=history_url %}
{% endblock %}

{% block form_action %}{{ view.edit_url }}{% endblock %}

{% block form_actions %}
    <div class="dropdown dropup dropdown-button match-width">
        <button type="submit" class="button action-save button-longrunning" data-clicked-text="{% trans 'Saving…' %}">
            {% icon name="spinner" %}<em>{% trans 'Save' %}</em>
        </button>

        {% if user_can_delete %}
            <div class="dropdown-toggle">{% icon name="arrow-up" %}</div>
            <ul>
                <li><a href="{{ view.delete_url }}" class="shortcut">{% trans "Delete" %}</a></li>
            </ul>
        {% endif %}

    </div>
{% endblock %}
