{% load i18n wagtailadmin_tags %}
<li data-inline-panel-child id="inline_child_{{ child.form.prefix }}" data-contentpath-disabled>
    {% if child.form.non_field_errors %}
        <ul>
            {% for error in child.form.non_field_errors %}
                <li class="error-message">
                    <span>{{ error|escape }}</span>
                </li>
            {% endfor %}
        </ul>
    {% endif %}
    <ul class="controls">
        {% if can_order %}
            <li><button type="button" class="button button-small button--icon text-replace white inline-child-move-up" id="{{ child.form.prefix }}-move-up" title="{% trans 'Move up' %}">{% icon name="order-up" %}{% trans "Move up" %}</button></li>
            <li><button type="button" class="button button-small button--icon text-replace white inline-child-move-down" id="{{ child.form.prefix }}-move-down" title="{% trans 'Move down' %}">{% icon name="order-down" %}{% trans "Move down" %}</button></li>
        {% endif %}
        <li><button type="button" class="button button-small button--icon text-replace white hover-no" id="{{ child.form.DELETE.id_for_label }}-button" title="{% trans 'Delete' %}">{% icon name="bin" %}{% trans "Delete" %}</button></li>
    </ul>
    {{ child.render_form_content }}
</li>
