{% load i18n wagtailadmin_tags %}
<li id="inline_child_{{ form.prefix }}"{% if form.DELETE.value %} style="display: none;"{% endif %}>
    <ul class="controls">
        <li><button type="button" class="button button--icon text-replace inline-child-move-up" id="{{ form.prefix }}-move-up">{% icon name="order-up" %}{% trans "Move up" %}</button></li>
        <li><button type="button" class="button button--icon text-replace inline-child-move-down" id="{{ form.prefix }}-move-down">{% icon name="order-down" %}{% trans "Move down" %}</button></li>
        <li><button type="button" class="button button--icon text-replace" id="{{ form.DELETE.id_for_label }}-button">{% icon name="bin" %}{% trans "Delete" %}</button></li>
    </ul>

    <fieldset>
        <legend>{% trans "Promoted search result" %}</legend>
        <ul class="fields">
            <li class="model_choice_field">
                {% include "wagtailadmin/shared/field.html" with field=form.page only %}
            </li>
            <li class="char_field">
                {% include "wagtailadmin/shared/field.html" with field=form.description only %}
            </li>
        </ul>
    </fieldset>
    {{ form.id }}
    {{ form.ORDER }}
    {{ form.DELETE }}
</li>
