# Generated by Django 3.0.10 on 2020-12-19 15:07

from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ("wagtailadmin", "0001_create_admin_access_permissions"),
    ]

    operations = [
        migrations.CreateModel(
            name="Admin",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
            ],
            options={
                "permissions": [("access_admin", "Can access Wagtail admin")],
                "managed": False,
                "default_permissions": [],
            },
        ),
    ]
