{% extends "wagtailadmin/base.html" %}
{% load i18n %}
{% block titletag %}{% blocktrans trimmed with title=page.get_admin_display_title %}Unpublish {{ title }}{% endblocktrans %}{% endblock %}
{% block content %}
    {% trans "Unpublish" as unpublish_str %}
    {% include "wagtailadmin/shared/header.html" with title=unpublish_str subtitle=page.get_admin_display_title icon="doc-empty-inverse" %}

    <div class="nice-padding">
        <p>{% trans "Are you sure you want to unpublish this page?" %}</p>
        <form action="{% url 'wagtailadmin_pages:unpublish' page.id %}" method="POST">
            {% csrf_token %}
            <input type="hidden" name="next" value="{{ next }}">
            <ul class="fields">
                {% if live_descendant_count > 0 %}
                    <li>
                        <div class="field boolean_field checkbox_input">
                            <div class="field-content">
                                <div class="input">
                                    <input id="id_include_descendants" name="include_descendants" type="checkbox">
                                    <label for="id_include_descendants" class="plain-checkbox-label">{% blocktrans trimmed count counter=live_descendant_count %}
                                        This page has one subpage. Unpublish this too
                                    {% plural %}
                                        This page has {{ live_descendant_count }} subpages. Unpublish these too
                                    {% endblocktrans %}</label>
                                </div>
                            </div>
                        </div>
                    </li>
                {% endif %}
                <li>
                    <input type="submit" value="{% trans 'Yes, unpublish it' %}" class="button">
                    <a href="{% if next %}{{ next }}{% else %}{% url 'wagtailadmin_explore' page.get_parent.id %}{% endif %}" class="button button-secondary">{% trans "No, don't unpublish" %}</a>
                </li>
            </ul>
        </form>
    </div>
{% endblock %}
