.page404__bg{background-color:var(--color-primary-darker);color:#fff;font-family:-apple-system,BlinkMacSystemFont,Segoe UI,system-ui,Roboto,Helvetica Neue,Arial,sans-serif,Apple Color Emoji,Segoe UI Emoji,Segoe UI Symbol,Noto Color Emoji;height:100vh;inset-inline-start:0;left:0;position:fixed;top:0;width:100vw}.page404__wrapper{align-items:center;display:flex;inset-inline-start:50%;justify-content:center;left:50%;position:absolute;top:50%;transform:translate(-50%,-50%)}.page404__logo{float:left;height:500px;width:400px}@media(forced-colors:active){.page404__logo{background-color:LinkText}}.page404__text-container{float:right;height:500px;text-align:center;width:600px}.page404__header{color:inherit;font-size:6.8em;margin-bottom:.2em}.page404__text{color:inherit;font-size:2.25em;line-height:1.25em}a.button.page404__button{background-color:var(--color-primary-darker);border:4px solid var(--color-primary);color:inherit;font-size:1.5em;height:2.5em;line-height:2em;padding:0 .5em}a.button.page404__button:hover{background-color:var(--color-primary)}@media screen and (max-width:49.9375em){.page404__text-container{width:400px}.page404__header{font-size:5em}}
