section{border-top:1px solid #d9d9d9;margin-top:2em;padding:0 0 2em}section>h2:first-child{background:#e6e6e6;font-size:1em;margin:0 0 1em;padding:1em}.palette:after,.palette:before{content:" ";display:table}.palette:after{clear:both}.palette ul{font-style:normal;list-style-type:none;margin-bottom:0;margin-top:0;padding-inline-start:0}.palette ul:after,.palette ul:before{content:" ";display:table}.palette ul:after{clear:both}.palette ul li{font-style:normal;list-style-type:none}.palette li{color:#000;float:left;height:100px;padding:10px;width:100px}.palette .contrast li{height:50px}.palette .contrast-large{font-size:18px}.palette .contrast-large li{height:55px;width:150px}.palette .color-black-text{color:#000}.palette .color-grey-1-text{color:#333}.palette .color-white-text{color:#fff}.palette .color-salmon-light-text{color:#fcf2f2}.palette .color-green-text{color:#189370}.palette .color-green-dark-text{color:#157b57}.palette .color-teal-darker-text{color:var(--color-primary-darker)}.palette .color-teal-dark-text{color:var(--color-primary-dark)}.palette .color-grey-2-text{color:#4d4d4d}.palette .color-red-text{color:#cd3238}.palette .color-red-dark-text{color:#b4191f}.palette .color-teal-text{color:var(--color-primary)}.palette .color-orange-text{color:#e9b04d}.palette .color-orange-dark-text{color:#bb5b03}.palette .color-blue-text{color:#71b2d4}.palette .color-salmon-text{color:#f37e77}.palette .color-grey-3-text{color:#d9d9d9}.palette .color-grey-4-text{color:#e6e6e6}.palette .color-grey-5-text{color:#fafafa}.palette .color-menu-text{color:#ccc}.palette .color-teal{background-color:var(--color-primary)}.palette .color-teal-darker{background-color:var(--color-primary-darker)}.palette .color-teal-dark{background-color:var(--color-primary-dark)}.palette .color-red{background-color:#cd3238}.palette .color-red-dark{background-color:#b4191f}.palette .color-orange{background-color:#e9b04d}.palette .color-orange-dark{background-color:#bb5b03}.palette .color-green{background-color:#189370}.palette .color-green-dark{background-color:#157b57}.palette .color-blue{background-color:#71b2d4}.palette .color-grey-1{background-color:#333}.palette .color-grey-2{background-color:#4d4d4d}.palette .color-grey-3{background-color:#d9d9d9}.palette .color-grey-4{background-color:#e6e6e6}.palette .color-grey-5{background-color:#fafafa}.palette .color-salmon{background-color:#f37e77}.palette .color-salmon-light{background-color:#fcf2f2}.icons :after,.icons :before{font-size:2em}.icons .icon{height:1.5em;vertical-align:text-top;width:1.5em}.icons ul{column-count:3}.icons li{margin-bottom:1em}.icons .spinner{position:relative}.timepicker{height:150px}
