.listing .field label{clip:rect(0 0 0 0);border:0;height:1px;margin:-1px;overflow:hidden;padding:0;position:absolute;width:1px}.listing input,.listing select,.listing textarea{font-size:1em}.listing select+span:after{font-size:2.5em!important}.listing .custom-permissions{padding-bottom:0}.listing .custom-permissions-item{font-weight:inherit;padding:0;width:100%}.page-permissions-listing .admin_page_chooser .field-content{width:100%}
