{% load l10n static %}
<script type="text/javascript"
    id="modeladmin-prepopulated-fields-constants"
    src="{% static "wagtailmodeladmin/js/prepopulate.js" %}"
    data-prepopulated-fields="{{ prepopulated_fields_json }}">
</script>
